require "net/smtp.rb"
require "base64"

################################################################################
#
#	Email
#
class Email

	attr_reader		:header
	attr_reader		:body
	attr_accessor	:smtpServer

	#---------------------------------------------------------------------------
	#
	#	Initialize ᡼륪֥Ȥ
	#
	def initialize(mail = nil)	# mail = Array or IO(ex.STDIN) or nil
		key     = nil
		@header = {}
		@body   = []

		return if mail == nil

		inBody = false
		mail.each {|line|
			line.chomp!
			unless inBody
				if line =~ /^$/					# 
					inBody = true
				elsif line =~ /^(\S+?):\s*(.*)/	# إå
					key = $1.capitalize
					@header[key] = $2
				elsif key						# إåԤ2ԤϤ
					@header[key] += "\n" + line.sub(/^\s*/, "\t")
				end
			else
				@body = mail
				break
			end
		}
	end

	#---------------------------------------------------------------------------
	#
	#	[] إå򻲾
	#
	def [](key)
		@header[key.capitalize]
	end

	#---------------------------------------------------------------------------
	#
	#	[]= إå
	#
	def []=(key, value)
		@header[key.capitalize] = value
	end

	#---------------------------------------------------------------------------
	#
	#	<< ܥǥ˥ƥȤɲ
	#
	def <<(message)
		@body.push message
	end

	#---------------------------------------------------------------------------
	#
	#	header_each ᡼إåϤ
	#
	def header_each
		@header.each {|key, value|
			yield("#{key}: #{value}\n")
		}
	end

	#---------------------------------------------------------------------------
	#
	#	body_each ᡼ܥǥϤ(ȥ꡼फ硢Τ1)
	#
	def body_each
		@body.each {|l|
			yield("#{l}\n")
		}
	end

	#---------------------------------------------------------------------------
	#
	#	each ᡼ǡϤ
	#
	def each
		self.header_each {|h|
			yield(h)
		}
		yield("\n")								# إå/ܥǥΥѥ졼
		self.body_each {|l|
			yield(l)
		}
	end

	#---------------------------------------------------------------------------
	#
	#	send ᡼
	#
	def send(from = nil, *to)
		from  = @header['From'] unless from
		to.push @header['To']   if to.size == 0
		Net::SMTP.start(smtpServer ? smtpServer : "localhost") {|smtp|
			smtp.send_mail(self, from, *to)
		}
	end
end

################################################################################
#
#	String
#
class String

	#---------------------------------------------------------------------------
	#
	#	mime encode
	#
	def mime_encode
		return "=?ISO-2022-JP?B?" + encode64(self.tojis).gsub!(/\n/, "") + "?="
	end

	#---------------------------------------------------------------------------
	#
	#	mime decode
	#
	def mime_decode
		return self unless self =~ /^=\?ISO-2022-JP\?B\?(.+)\?=$/i
		return decode64($1)
	end
end

################################################################################
#
#	EncodedEmail
#
class EncodedEmail < Email

	#---------------------------------------------------------------------------
	#
	#	<< ܥǥ˥ƥȤɲ(IO󥹥󥹤Ϥ)
	#
	def <<(io)
		@body = io
	end

	#---------------------------------------------------------------------------
	#
	#	body_each ᡼ܥǥϤ(ȥ꡼फ硢Τ1)
	#
	def body_each
		if self['Content-transfer-encoding'] =~ /base64/i
			@body.each {|l|
				yield(decode64(l))
			}
		else
			super
		end
	end

	#---------------------------------------------------------------------------
	#
	#	each ᡼ǡϤ
	#
	def each
		if self['Content-transfer-encoding'] =~ /base64/i
			self.header_each {|l|
				yield(l)
			}
			yield("\n")							# إå/ܥǥΥѥ졼
			while(l = @body.read(45))
				yield(encode64(l))
			end
		else
			super
		end
	end
end

################################################################################
#
#	AttachedEmail
#
class AttachedEmail < EncodedEmail

	attr_reader	:block

	#---------------------------------------------------------------------------
	#
	#	Initialize źե᡼륪֥Ȥ
	#
	def initialize(mail = nil)

		super(mail)

		if mail == nil
			@separator = "separator" + (rand 65536).to_s
			self['MIME-Version'] = "1.0"
			self['Content-Type'] = "Multipart/Mixed; boundary=\"#{@separator}\""
			@block = []
			return
		end

		return unless self['Content-Type'] =~ /^Multipart\/Mixed;\s*boundary=(["']?)(.*)\1/i
		@separator = $2
		@block = nil
	end

	#---------------------------------------------------------------------------
	#
	#	block_each ᡼֥åϤ
	#
	def block_each
		if(@block)
			@block.each {|b|
				yield(b)
			}
		else
			def @body.each						# each򥻥ѥ졼ǻߤޤðۥ᥽åɤѹ
				super {|l|
					break if l =~ /^--#{@separator}/
					yield(l)
				}
			end
			loop {
				break if(@body.eof)
				yield(EncodedEmail.new(@body))
			}
		end
	end

	#---------------------------------------------------------------------------
	#
	#	<< ᡼֥åɲ
	#
	def <<(block)
		@block.push(block)
	end

	#---------------------------------------------------------------------------
	#
	#	each ᡼ǡϤ
	#
	def each
		self.header_each {|h|
			yield(h)
		}
		yield("\n")								# إå/ܥǥΥѥ졼
		@block.each {|b|
			yield("--" + @separator + "\n")
			b.each {|l|
				yield l
			}
		}
		yield("--" + @separator + "--\n")
	end
end
